require 'sinatra/base'
require 'json'

require 'useless/doc/server/sinatra'
require 'useless/rack/files'

module Useless
  class Museum < Sinatra::Base
    require 'useless/museum/image'
    require 'useless/museum/version'

    use Useless::Rack::Files

    register Useless::Doc::Server::Sinatra

    enable :raise_errors
    disable :show_exceptions, :dump_errors
    set :logging, nil

    doc 'Museum' do
      url 'http://museum.useless.io'

      description <<-DESC
        The Museum of Whatever
      DESC
    end

    get '/' do
      redirect 'http://museum.doc.useless.io'
    end

    doc.get '/photos/:id' do
      description 'Retrieve information about a photo.'
      authentication_required false
      parameter 'id', 'The ID of the desired photo.', type: 'path'

      response 404, 'A photo with the specified ID could not be found.'
      response 200, 'The photo was returned successfully.' do
        body do
          content_type 'application/json'
          attribute 'id', 'The ID of the photo.'
          attribute 'file_urls',
            ['An object containing each of files of the photo\'s sizes. "small" has a',
             'longest side of 100 pixels, "medium" has a longest side of 500 pixels and',
             '"large" has a longest side of 1024 pixels.'].join("\n"), type: 'object'
          attribute 'latitude', 'The latitude where the photo was shot.', type: 'number'
          attribute 'longitude', 'The longitude where the photo was shot.', type: 'number'
          attribute 'shot_at', 'The date and time when the photo was shot.'
          attribute 'description', 'A description of the photo provided by the owner.'
          attribute 'created_by', 'The user who created the photo.', type: 'object'
          attribute 'created_at', 'When the photo was created.'
        end
      end
    end

    get '/photos/:id' do
      return 404 unless BSON::ObjectId.legal?(params[:id])

      id = BSON::ObjectId.from_string(params[:id])
      if record = env['useless.mongo']['museum.photos'].find_one(id)
        photo_json(record)
      else
        404
      end
    end

    doc.post '/photos' do
      description 'Create a new photo.'
      authentication_required true

      body do
        content_type 'multipart/form-data'
        attribute 'photo', 'The actual file for the photo.', type: 'file'
        attribute 'latitude',
          ['The latitude where the photo was taken. Only required if it cannot be',
           'extracted from the photo itself. If the information can be extracted,',
           'this will be ignored.'].join("\n"),
          required: false, type: 'number', default: 'value extracted from photo'
        attribute 'longitude', 'The longitude where the photo was taken. Follows the same rules as latitude.',
          required: false, type: 'number', default: 'value extracted from photo'
        attribute 'shot_at', 'The time that the photo was taken. Follows the same rules as latitude and longitude.',
          required: false, default: 'value extracted from photo'
        attribute 'description', 'The description of the photo.', required: false, default: nil
      end

      response 201, 'The photo was created successfully.' do
        header 'Location', 'The URL of the new photo.'
      end
      response 401, 'The request could not be authenticated.'
      response 422, 'latitude, longitude or shot_at could not be extracted from the photo. ' +
                    'Retry with the latitude, longitude and shot_at attributes specified.'
      response 422, 'Invalid image format. Format must be one of JPEG, TIFF, PNG or GIF.'
    end

    post '/photos' do
      unless env['useless.user']
        halt 401, 'The request could not be authenticated.'
      end

      unless params[:file]
        halt 422, 'File is a required parameter.'
      end

      file = params[:file][:tempfile]
      image = Image.new(file)

      unless image.valid?
        halt 422, 'Invalid image format. Format must be one of JPEG, TIFF, PNG or GIF.'
      end

      document = {}
      document['latitude'] = image.latitude || params[:latitude]
      document['longitude'] = image.longitude || params[:longitude]
      document['shot_at'] = image.shot_at || (params[:shot_at] && Time.parse(params[:shot_at]))

      unless document['latitude'] and document['longitude'] and document['shot_at']
        halt 422, 'latitude, longitude or shot_at could not be extracted from the photo. ' +
                  'Retry with the latitude, longitude and shot_at attributes specified.'
      end

      document['file_url'] = {
        'small' =>  create_image_url(image.small),
        'medium' => create_image_url(image.medium),
        'large' =>  create_image_url(image.large)
      }
      document['description'] = params[:description]
      document['created_by_id'] = env['useless.user']['_id']
      document['created_at'] = document['updated_at'] = Time.now
      photo_id = env['useless.mongo']['museum.photos'] << document

      status 201
      headers 'Location' => "http://museum.useless.io/photos/#{photo_id}"
      content_type 'application/json'
      photo_json(document)
    end

    def photo_json(raw_record)
      record = raw_record.dup

      record['id'] = record.delete('_id').to_s

      record['shot_at'] = raw_record['shot_at'].iso8601
      record['created_at'] = raw_record['created_at'].iso8601
      record['updated_at'] = raw_record['updated_at'].iso8601

      created_by_id = record.delete('created_by_id')
      created_by = env['useless.mongo']['users'].find_one(created_by_id)
      record['created_by'] = {'id' => created_by_id.to_s, 'handle' => created_by['handle']}

      record.to_json
    end

    def create_image_url(image)
      id = env['useless.fs'].put(image.to_blob, content_type: image.mime_type)
      "http://museum.useless.io/files/#{id}"
    end
  end
end
