require 'mini_magick'
require 'exifr'

module Useless
  class Museum
    # Museum::Image is a utility class that provides two types of
    # functionality:
    #   * image resizing based upon the Useless::Musuem specifications
    #   * access to the following image metadata: latitude, longitude, shot_at
    # It takes an IO object upon initialization. The resize methods return
    # instances of MiniMagick::Image.
    class Image
      def initialize(raw_io)
        @raw_io = raw_io
      end

      # MiniMagick::Image for the original image
      def base
        @base ||= minimagick_copy
      end

      # Only JPEGs are considered valid
      def valid?
        base and ['JPEG', 'TIFF', 'PNG', 'GIF'].include?(base['format'])
      end

      # A 'small' image's longest side is 100 pixels,
      def small
        @small ||= version('100x100')
      end

      # 'medium' is 500 pixels
      def medium
        @medium ||= version('500x500')
      end

      # and 'large' is 1024 pixels
      def large
        @large ||= version('1024x1024')
      end

      # Just grab latitude
      def latitude
        @latitude ||= exifr.gps.latitude if exifr and exifr.gps
      end

      # and longitude from EXIFR
      def longitude
        @longitude ||= exifr.gps.longitude if exifr and exifr.gps
      end

      # Shot at is just the date_time provide from EXIFR
      def shot_at
        @shot_at ||= exifr.date_time if exifr
      end

      private

      # An EXIFR::JPEG instance, which provides access to the image metadata.
      def exifr
        return nil unless base

        @exifr ||= case base['format']
          when 'JPEG' then EXIFR::JPEG.new(StringIO.new(blob))
          when 'TIFF' then EXIFR::TIFF.new(StringIO.new(blob))
          else nil
          end
      end

      def version(dimensions)
        # Copy the image if possible,
        if image = minimagick_copy

          # resize it to the specified dimensions
          image.resize dimensions

          # and return
          image
        end
      end

      def minimagick_copy
        # Just instantiate a new instance with the raw image blob.
        MiniMagick::Image.read(blob)

        # If the 'image' cannot be parsed,
      rescue MiniMagick::Invalid

        # just return nil
        nil
      end

      # A binary blob of the original image
      def blob
        @blob ||= begin
          # Make sure the pointer is up front,
          @raw_io.rewind

          # read the bytes,
          blob = @raw_io.read

          # make sure they're un-encoded (encodings can cause multiple bytes to
          # be interpreted as one, which is no good for images),
          blob.force_encoding('BINARY')

          # and return
          blob
        end
      end
    end
  end
end
